SystemOrganization addCategory: #UnixMetaStack!

StackInterpreterPrimitives subclass: #MetaStackInterpreter
	instanceVariableNames: ''
	classVariableNames: 'AllButExperimentalBit ExperimentalObjectBit'
	poolDictionaries: ''
	category: 'UnixMetaStack'!

!MetaStackInterpreter class methodsFor: 'common selector sends' stamp: 'NickPapoulias 3/13/2014 19:59'!
enableOverrideOfByteCodes

	((StackInterpreter methods 
		select: [ :m | m protocol = #'common selector sends' ]) 
			collect: [ :m | m selector asString , ' self normalSend']) do: [:m |
				self compile: m			
	]! !

!MetaStackInterpreter class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/5/2013 19:27'!
initialize

	super initialize.
	BytesPerWord ifNil: [BytesPerWord := 4].  "May get called on fileIn, so supply default"	
	WordMask := (1 bitShift: BytesPerWord*8) - 1.
	ExperimentalObjectBit := 1 bitShift: BytesPerWord *8 - 3.  "Next-to-Top bit"
	AllButExperimentalBit  := WordMask - ExperimentalObjectBit.! !

!MetaStackInterpreter class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/6/2013 12:07'!
initializePrimitiveTable

	super initializePrimitiveTable.
	
	PrimitiveTable at: 150 + 1 put: #primitiveGetExperimentalBit.
	PrimitiveTable at: 151 + 1 put: #primitiveSetExperimentalBit.
	PrimitiveTable at: 152 + 1 put: #primitiveTurnOffExperimentalBitOfAllObjects.! !

!MetaStackInterpreter methodsFor: 'meta-protocol-overrides-placeholders' stamp: 'NickPapoulias 5/28/2013 13:28'!
commonReturn
	"Do an ^-return (return form method), perhaps checking for unwinds if this is a block activation.
	 Note: Assumed to be inlined into the dispatch loop."

	super commonReturn! !

!MetaStackInterpreter methodsFor: 'meta-protocol-overrides' stamp: 'NickPapoulias 3/15/2014 20:04'!
commonSend
	"Send a message, starting lookup with the receiver's class."
	"Assume: messageSelector and argumentCount have been set, and that 
	the receiver and arguments have been pushed onto the stack,"
	"Note: This method is inlined into the interpreter dispatch loop."
	<sharedCodeNamed: 'commonSend' inCase: #singleExtendedSendBytecode>
		
	(objectMemory splObj: 79) = objectMemory falseObject ifTrue: [ ^ super commonSend ].	
		
	(objectMemory splObj: 63) == (self receiver) ifTrue: [ "sender is Interpreter"
		^ super commonSend.
	].

	(objectMemory splObj: 63) == (self internalStackValue: argumentCount ) ifTrue: ["receiver is Interpreter -- this is from wherever"
		^ super commonSend.
	].
	
	(objectMemory splObj: 65) == (self receiver) ifTrue: [ "sender is the baseMetaDict"
		^ super commonSend.
	].

	(self shouldOverrideBehaviorFor: self receiver) ifTrue: [ "override for sender"
		^ self sendMetaSendFor: self receiver.
	].

	(self shouldOverrideBehaviorFor: (self internalStackValue: argumentCount)) ifTrue: [ "override fo receiver"
		^ self sendMetaReceivedFor: (self internalStackValue: argumentCount).
	].
	

	
	
	0 to: argumentCount - 1 do: [:argIndex | "override for args" 
		 (self shouldOverrideBehaviorFor: (self internalStackValue: argIndex)) ifTrue: [
					^ self sendMetaArgFor: (self internalStackValue: argIndex)
		]
	].
	
	(self shouldOverrideBehaviorFor: method) ifTrue: [
		"should distinguish between method as object and method as method"
		^ self sendMetaMSendFor: method.	
	].
	
	self sendBreak: messageSelector + BaseHeaderSize
		point: (objectMemory lengthOf: messageSelector)
		receiver: (self internalStackValue: argumentCount).
	self printSends ifTrue:
		[self printActivationNameForSelector: messageSelector startClass: lkupClass; cr].
	self internalFindNewMethod.
	(self shouldOverrideBehaviorFor: newMethod) ifTrue: [
		^ self sendMetaInvokedFor: newMethod.	
	].
	self internalExecuteNewMethod.
	self fetchNextBytecode
	
	! !

!MetaStackInterpreter methodsFor: 'meta-protocol-overrides' stamp: 'NickPapoulias 5/9/2013 15:39'!
externalQuickPrimitiveResponse
	"Invoke a quick primitive.
	 Called under the assumption that primFunctionPtr has been preloaded"
	
	| localPrimIndex array |
	self assert: self isPrimitiveFunctionPointerAnIndex.
	localPrimIndex := self cCoerceSimple: primitiveFunctionPointer to: #sqInt.
	self assert: (localPrimIndex > 255 and: [localPrimIndex < 520]).
	"Quick return inst vars"
	localPrimIndex >= 264 ifTrue: [
		(self shouldOverrideBehaviorFor: self stackTop) ifTrue: [
			self unsetOverrideBehaviorFor: self stackTop.
			array := objectMemory eeInstantiateClass: (objectMemory splObj: ClassArray) indexableSize: 1.
			objectMemory storePointerUnchecked: 0 ofObject: array withValue: (self integerObjectOf: (localPrimIndex - 264 + 1)).
			self primitiveObject: self stackTop
				perform: (objectMemory splObj: 60) 
				withArguments:  array
				lookedUpIn: (objectMemory fetchClassOf: self stackTop).
			^ true.
		] ifFalse: [
			self pop: 1 thenPush: (objectMemory fetchPointer: localPrimIndex - 264 ofObject: self stackTop).
		 	^true
		].
	].
	"Quick return constants"
	localPrimIndex = 256 ifTrue: [^true "return self"].
	localPrimIndex = 257 ifTrue: [self pop: 1 thenPush: objectMemory trueObject. ^true].
	localPrimIndex = 258 ifTrue: [self pop: 1 thenPush: objectMemory falseObject. ^true].
	localPrimIndex = 259 ifTrue: [self pop: 1 thenPush: objectMemory nilObject. ^true].
	self pop: 1 thenPush: (objectMemory integerObjectOf: localPrimIndex - 261).
	^true
! !

!MetaStackInterpreter methodsFor: 'experimental-bit-accesors' stamp: 'NickPapoulias 4/30/2013 17:58'!
getExperimentalBitOf: anOop
	self inline: true.
	^((objectMemory baseHeader: anOop) bitAnd: ExperimentalObjectBit) ~= 0 ! !

!MetaStackInterpreter methodsFor: 'experimental-bit-primitives' stamp: 'NickPapoulias 5/7/2013 15:58'!
primitiveGetExperimentalBit
	| rcvr hdr |
	rcvr := self stackTop.
	(self isIntegerObject: rcvr)
		ifTrue:
			[self primitiveFail ]
		ifFalse:
			[ self pop: 2. 
			self pushBool: (self getExperimentalBitOf: rcvr) ]! !

!MetaStackInterpreter methodsFor: 'experimental-bit-primitives' stamp: 'NickPapoulias 5/7/2013 15:28'!
primitiveSetExperimentalBit
	| boolean rcvr hdr wasImmutable |
	rcvr := self stackValue: 1.
	(self isIntegerObject: rcvr) ifTrue:
		[^self primitiveFailFor: PrimErrInappropriate].
	boolean := self booleanValueOf: self stackTop.
	self successful ifFalse:
		[^self primitiveFailFor: PrimErrBadArgument].
	self setExperimentalBitOf: rcvr to: boolean.
	self pop: 2! !

!MetaStackInterpreter methodsFor: 'experimental-bit-primitives' stamp: 'NickPapoulias 5/5/2013 19:07'!
primitiveTurnOffExperimentalBitOfAllObjects
	| oop |
	oop := self firstAccessibleObject.
	[oop = nil] whileFalse: [
		(self isIntegerObject:  oop)
			ifFalse: [ 
				self setExperimentalBitOf: oop to: false
				].
		oop := self accessibleObjectAfter: oop.
	].
	
! !

!MetaStackInterpreter methodsFor: 'print-debugging' stamp: 'NickPapoulias 4/30/2013 18:01'!
print: s
	
	<api>
	<var: #s type: #'char *'>
	self cCode: 'fputs(s, stdout); fflush(stdout);'! !

!MetaStackInterpreter methodsFor: 'meta-protocol-overrides-placeholders' stamp: 'NickPapoulias 5/5/2013 17:04'!
pushLiteralVariable: literalIndex

	"place holder for hooking metaRef if needed"

	super pushLiteralVariable: literalIndex.
! !

!MetaStackInterpreter methodsFor: 'meta-protocol-overrides-placeholders' stamp: 'NickPapoulias 5/5/2013 17:05'!
pushReceiverBytecode

	"place holder for hooking metaRef if needed"

	super pushReceiverBytecode.
! !

!MetaStackInterpreter methodsFor: 'meta-protocol-overrides' stamp: 'NickPapoulias 5/5/2013 17:05'!
pushReceiverVariable: fieldIndex

	|iv object|

	self inline: true.
	
	object := self receiver.
	
	(self shouldOverrideBehaviorFor: object) ifTrue: [
		self sendMetaReadAtFor: object withIndex: (self integerObjectOf: fieldIndex + 1). 
	] ifFalse: [
		super pushReceiverVariable: fieldIndex.
	].
	
	
! !

!MetaStackInterpreter methodsFor: 'meta-protocol-overrides-placeholders' stamp: 'NickPapoulias 5/5/2013 17:06'!
pushTemporaryVariable: temporaryIndex

	super pushTemporaryVariable: temporaryIndex.

	
! !

!MetaStackInterpreter methodsFor: 'meta-protocol' stamp: 'NickPapoulias 5/29/2013 16:42'!
sendMetaArgFor: object

	| message rcvr |
	self inline: true.

	self unsetOverrideBehaviorFor: object.
	rcvr := self internalStackValue: argumentCount .
	self externalizeIPandSP.
	self createActualMessageTo: lkupClass.
	message := self stackTop.
	self internalizeIPandSP.
	self internalPop: 2.
	self internalPush: (objectMemory splObj: 63).
	self internalPush: object.
	self internalPush: message. "push the interpreter which is the receiver"
	self internalPush: rcvr.
	messageSelector := objectMemory splObj: 75. 
	argumentCount := 3. 
	self normalSend.! !

!MetaStackInterpreter methodsFor: 'meta-protocol' stamp: 'NickPapoulias 3/13/2014 08:42'!
sendMetaInvokedFor:  object	

	| rcvr message |
	self inline: true.

	self unsetOverrideBehaviorFor: object.
	rcvr := self internalStackValue: argumentCount .
	self externalizeIPandSP.
	self createActualMessageTo: lkupClass.
	message := self stackTop.
	self internalizeIPandSP.
	self internalPop: 2.
	self internalPush: (objectMemory splObj: 63).
	self internalPush: object.
	self internalPush: message.
	self internalPush: rcvr.
	messageSelector := objectMemory splObj: 77.
	argumentCount := 3. 
	self normalSend.! !

!MetaStackInterpreter methodsFor: 'meta-protocol' stamp: 'NickPapoulias 3/13/2014 13:18'!
sendMetaMSendFor:  object	

	| rcvr message |
	self inline: true.

	self unsetOverrideBehaviorFor: object.
	rcvr := self internalStackValue: argumentCount .
	self externalizeIPandSP.
	self createActualMessageTo: lkupClass.
	message := self stackTop.
	self internalizeIPandSP.
	self internalPop: 2.
	self internalPush: (objectMemory splObj: 63).
	self internalPush: object.
	self internalPush: message.
	self internalPush: rcvr.
	messageSelector := objectMemory splObj: 78.
	argumentCount := 3. 
	self normalSend.! !

!MetaStackInterpreter methodsFor: 'meta-protocol' stamp: 'NickPapoulias 5/29/2013 15:12'!
sendMetaReadAtFor: object withIndex: index

	"self inline: true."

	self unsetOverrideBehaviorFor: object.
	self internalPush: (objectMemory splObj: 63).
	self internalPush: object.
	self internalPush: index.
	messageSelector := objectMemory splObj: 73.
	argumentCount := 2.
	self undoFetchNextBytecode. "hail marry hack"
	self normalSend.! !

!MetaStackInterpreter methodsFor: 'meta-protocol' stamp: 'NickPapoulias 5/29/2013 15:02'!
sendMetaReceivedFor: object

	| message |
	self inline: true.

	self unsetOverrideBehaviorFor: object.
	self externalizeIPandSP.
	self createActualMessageTo: lkupClass.
	message := self stackTop.
	self internalizeIPandSP.
	self internalPop: 2.
	self internalPush: (objectMemory splObj: 63).
	self internalPush: object.
	self internalPush: message. "push the interpreter which is the receiver"
	argumentCount := 2.
	messageSelector := objectMemory splObj: 71. 
	self normalSend.! !

!MetaStackInterpreter methodsFor: 'meta-protocol' stamp: 'NickPapoulias 4/30/2013 18:16'!
sendMetaReferenceFor: object 

	self inline: true.
	
	self unsetOverrideBehaviorFor: object.
	self internalPush: object.
	messageSelector := objectMemory splObj: 59.
	argumentCount := 0.
	self normalSend.! !

!MetaStackInterpreter methodsFor: 'meta-protocol' stamp: 'NickPapoulias 5/29/2013 16:24'!
sendMetaSendFor: object

	| rcvr message |
	self inline: true.

	self unsetOverrideBehaviorFor: object.
	rcvr := self internalStackValue: argumentCount .
	self externalizeIPandSP.
	self createActualMessageTo: lkupClass.
	message := self stackTop.
	self internalizeIPandSP.
	self internalPop: 2.
	self internalPush: (objectMemory splObj: 63).
	self internalPush: object.
	self internalPush: message.
	self internalPush: rcvr.
	messageSelector := objectMemory splObj: 72.
	argumentCount := 3. 
	self normalSend.! !

!MetaStackInterpreter methodsFor: 'meta-protocol' stamp: 'NickPapoulias 6/17/2013 10:39'!
sendMetaStoreFor: object inIndex: index ofObject: anotherObject

	self inline: true.

	self unsetOverrideBehaviorFor: object.
	self internalPush: (objectMemory splObj: 63).
	self internalPush: object.
	self internalPush: index.
	self internalPush: anotherObject.
	messageSelector := objectMemory splObj: 76.
	argumentCount := 3.
	self normalSend.! !

!MetaStackInterpreter methodsFor: 'meta-protocol' stamp: 'NickPapoulias 5/29/2013 15:15'!
sendMetaWriteAtPutFor: object withIndex: index andValue: value 

	self inline: true.

	self unsetOverrideBehaviorFor: object.
	self internalPush: (objectMemory splObj: 63).
	self internalPush: object.
	self internalPush: index.
	self internalPush: value.
	messageSelector := objectMemory splObj: 74.
	argumentCount := 3.
	self normalSend.! !

!MetaStackInterpreter methodsFor: 'experimental-bit-accesors' stamp: 'NickPapoulias 4/30/2013 17:59'!
setExperimentalBitOf: anOop to: boolean
	| hdr |
	self inline: true.
	"Dont check here if it is integer. Check in the sender of this."
	hdr := objectMemory baseHeader: anOop.
	boolean 
		ifTrue: [ objectMemory baseHeader: anOop put: (hdr bitOr: ExperimentalObjectBit). ]
		ifFalse: [ objectMemory baseHeader: anOop put: (hdr bitAnd: AllButExperimentalBit). ]! !

!MetaStackInterpreter methodsFor: 'meta-jump-accesors' stamp: 'NickPapoulias 5/5/2013 17:09'!
setOverrideBehaviorFor: object

	self inline: true.
	
	objectMemory baseHeader: object put: ((objectMemory baseHeader: object) bitOr: (ExperimentalObjectBit))! !

!MetaStackInterpreter methodsFor: 'meta-jump-accesors' stamp: 'NickPapoulias 5/5/2013 17:09'!
shouldOverrideBehaviorFor: object
	
	self inline: true.
	
	^ (objectMemory isIntegerObject: object) not  & (objectMemory getExperimentalBitOf: object)! !

!MetaStackInterpreter methodsFor: 'meta-protocol-overrides' stamp: 'NickPapoulias 6/17/2013 10:14'!
storeAndPopReceiverVariableBytecode

	| rcvr top |

	self inline: true.

	rcvr := self receiver.
	top := self internalStackTop.

	(self shouldOverrideBehaviorFor: rcvr) ifTrue: [
		^ self sendMetaWriteAtPutFor: rcvr withIndex: (self integerObjectOf: (currentBytecode bitAnd: 7) + 1) andValue: top.
	]. 

	(self shouldOverrideBehaviorFor: top) ifTrue: [
		^ self sendMetaStoreFor: top inIndex: (self integerObjectOf: (currentBytecode bitAnd: 7) + 1) ofObject: rcvr.	
	]. 
	
	super storeAndPopReceiverVariableBytecode.
	
! !

!MetaStackInterpreter methodsFor: 'meta-protocol-overrides-placeholders' stamp: 'NickPapoulias 5/28/2013 13:26'!
storeAndPopTemporaryVariableBytecode
	
	super storeAndPopTemporaryVariableBytecode 
	
! !

!MetaStackInterpreter methodsFor: 'meta-jump-accesors' stamp: 'NickPapoulias 5/5/2013 17:09'!
unsetOverrideBehaviorFor: object
	
	self inline: true.
	
	objectMemory baseHeader: object put: ((objectMemory baseHeader: object) bitAnd: (AllButExperimentalBit)).! !

StackInterpreterPrimitives subclass: #ProfileStackInterpreter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'UnixMetaStack'!

ProfileStackInterpreter subclass: #ExtendedProfileStackInterpreter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'UnixMetaStack'!

!ExtendedProfileStackInterpreter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/12/2014 14:37'!
fetchNextBytecode

	"This method fetches the next instruction (bytecode). Each bytecode method is responsible for fetching the next bytecode, preferably as early as possible to allow the memory system time to process the request before the next dispatch."
	
	"bytecode counters -- for analysis see p. 595 of blue book and StackInterpreter>>initializeBytecodeTableForSqueakV3PlusClosures"
	self incByteCodeCounter.
	self cppIf: MULTIPLEBYTECODESETS
		ifTrue: [currentBytecode := self fetchByte + bytecodeSetSelector]
		ifFalse: [currentBytecode := self fetchByte].! !

!ExtendedProfileStackInterpreter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/12/2014 14:37'!
incByteCodeCounter

	<api>
	self cCode: 'bCounter++;bArrayCounter[getCurrentBytecode()]++;'! !

!ExtendedProfileStackInterpreter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/7/2014 18:19'!
internalExecuteNewMethod
	"regardless of whether it succeeded or not count it since it is executed"
	"for primitive Stats check StackInterpreter>>initializePrimitiveTable and StackInterpreter>>functionPointerFor: primIdx inClass: theClass"
	| succeeded |
	<inline: true>
	primitiveFunctionPointer ~= 0 ifTrue:
		[self incPrimitiveCounter.
		 self isPrimitiveFunctionPointerAnIndex ifTrue:
			[self profileQuickPrimitive. ^self internalQuickPrimitiveResponse].
		 self profileSlowPrimitive.
		 self externalizeIPandSP.
		 succeeded := self slowPrimitiveResponse.
		 self internalizeIPandSP.
		 succeeded ifTrue:
			[self browserPluginReturnIfNeeded.
			^nil]].
	"if not primitive, or primitive failed, activate the method"
	^self internalActivateNewMethod! !

!ExtendedProfileStackInterpreter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/12/2014 14:28'!
printProfilingResults

	<api>
	self cCode: 'unsigned int i;'.
	self cCode: 'printf("__VM_STATS__START__{ \
	#byteCodeCounter ->%llu. \
	#messageCounter -> %llu. \
	#primitiveCounter -> %llu. \
	#byteCode -> #( \
	",bCounter,mCounter,pCounter);'.
	self cCode: 'for(i=0;i<255;i++) printf("%llu\n",bArrayCounter[i])'.
	self  cCode: 'printf("). \
	#quickPrimitive -> #( \
	");'.
	self cCode: 'for(i=0;i<255;i++) printf("%llu\n",quickPrimitiveCounter[i])'.
	self cCode: 'printf("). \
	#slowPrimitive -> #( \
	");'.
	self cCode: 'for(i=0;i<255;i++) printf("%llu\n",slowPrimitiveCounter[i])'.
	self cCode: 'printf("). \
	}.__VM_STATS__END__ \
	");'.! !

!ExtendedProfileStackInterpreter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/7/2014 18:28'!
profileQuickPrimitive

	<api>
	|index|
	index := (self cCoerce: primitiveFunctionPointer to: 'unsigned long').
	self cCode: 'quickPrimitiveCounter[index]++;'. "check if naming persists in translation"! !

!ExtendedProfileStackInterpreter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/7/2014 20:14'!
profileSlowPrimitive

	<api>
	|index|
	index := -1.
	1 to: MaxPrimitiveIndex do: [ :i | (primitiveTable at: i) == primitiveFunctionPointer ifTrue: [ index := i ]  ].
	self cCode: 'if(index > 0){slowPrimitiveCounter[index]++;}'. "check if naming persists in translation"! !

!ProfileStackInterpreter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/5/2014 11:21'!
commonSend
	"Send a message, starting lookup with the receiver's class."
	"Assume: messageSelector and argumentCount have been set, and that 
	the receiver and arguments have been pushed onto the stack,"
	"Note: This method is inlined into the interpreter dispatch loop."
	<sharedCodeNamed: 'commonSend' inCase: #singleExtendedSendBytecode>
	self incMessageCounter.
	super commonSend.! !

!ProfileStackInterpreter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/5/2014 11:21'!
fetchNextBytecode
	"This method fetches the next instruction (bytecode). Each bytecode method is responsible for fetching the next bytecode, preferably as early as possible to allow the memory system time to process the request before the next dispatch."
	
	self incByteCodeCounter.
	self cppIf: MULTIPLEBYTECODESETS
		ifTrue: [currentBytecode := self fetchByte + bytecodeSetSelector]
		ifFalse: [currentBytecode := self fetchByte]! !

!ProfileStackInterpreter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/5/2014 11:20'!
incByteCodeCounter

	<api>
	self cCode: 'bCounter++;'! !

!ProfileStackInterpreter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/5/2014 11:20'!
incMessageCounter
	<api>
	self cCode: 'mCounter++;'! !

!ProfileStackInterpreter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/5/2014 11:20'!
incPrimitiveCounter
	<api>
	self cCode: 'pCounter++;'! !

!ProfileStackInterpreter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/5/2014 11:19'!
internalExecuteNewMethod
	"regardless of whether it succeeded or not count it since it is executed"
	| succeeded |
	<inline: true>
	primitiveFunctionPointer ~= 0 ifTrue:
		[self incPrimitiveCounter.self isPrimitiveFunctionPointerAnIndex ifTrue:
			[^self internalQuickPrimitiveResponse].
		 self externalizeIPandSP.
		 succeeded := self slowPrimitiveResponse.
		 self internalizeIPandSP.
		 succeeded ifTrue:
			[self browserPluginReturnIfNeeded.
			^nil]].
	"if not primitive, or primitive failed, activate the method"
	^self internalActivateNewMethod! !

!ProfileStackInterpreter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/5/2014 11:22'!
primitiveQuit
	self printProfilingResults.
	super primitiveQuit.! !

!ProfileStackInterpreter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/5/2014 10:08'!
print: s
	"For testing in Smalltalk, this method should be overridden in a subclass."
	<api>
	<var: #s type: #'char *'>
	self cCode: 'fputs(s, stdout)'! !

!ProfileStackInterpreter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/12/2014 13:57'!
printProfilingResults
	<api>
	self cCode: 'printf("__VM_STATS__START__{ \
	#byteCodeCounter ->%llu. \
	#messageCounter -> %llu. \
	#primitiveCounter -> %llu. }.__VM_STATS__END__ \
	",bCounter,mCounter,pCounter);'.
	! !

StackUnixDebugConfig subclass: #MetaStackUnixDebugConfig
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'UnixMetaStack'!

!MetaStackUnixDebugConfig methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/7/2013 10:05'!
executableName
	^ 'MetaStackDebugVM'! !

!MetaStackUnixDebugConfig methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/6/2013 16:13'!
interpreterClass
	^ MetaStackInterpreter! !

StackUnixDebugConfig subclass: #ProfileUnixDebugConfig
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'UnixMetaStack'!

!ProfileUnixDebugConfig methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/5/2014 10:25'!
configH
	" right now its like  that "
	^ '
#ifndef __sq_config_h
#define __sq_config_h

/* explicit image width */

#define	HAVE_INTERP_H 1

/* package options */

#define	USE_X11 1
#define	USE_X11_GLX 1
/* #undef	USE_QUARTZ */
/* #undef	USE_QUARTZ_CGL */
/* #undef	USE_RFB */

/* libraries */

/* #undef	HAVE_LIBX11 */
/* #undef	HAVE_LIBXEXT */
#define	HAVE_LIBDL 1
/* #undef	HAVE_DYLD */
/* #undef	HAVE_LIBFFI */
/* #undef	HAVE_ICONV */

/* #undef	USE_AUDIO_NONE */
/* #undef	USE_AUDIO_SUN */
/* #undef	USE_AUDIO_NAS */
/* #undef	USE_AUDIO_OSS */
/* #undef	USE_AUDIO_MACOSX */
/* #undef	OSS_DEVICE */

/* header files */

#define	HAVE_UNISTD_H 1
/* #undef	NEED_GETHOSTNAME_P */

#define	HAVE_DIRENT_H 1
/* #undef	HAVE_SYS_NDIR_H */
/* #undef	HAVE_SYS_DIR_H */
/* #undef	HAVE_NDIR_H */
#define	HAVE_DLFCN_H 1
#define	HAVE_ICONV_H 1

#define	HAVE_SYS_TIME_H 1
#define	TIME_WITH_SYS_TIME 1

/* #undef	HAVE_SYS_FILIO_H */

/* #undef	HAVE_SYS_AUDIOIO_H */
/* #undef	HAVE_SUN_AUDIOIO_H */

#define	HAVE_PTY_H 1
/* #undef	HAVE_UTIL_H */
/* #undef	HAVE_LIBUTIL_H */
#define	HAVE_STROPTS_H 1

#define	HAVE_GL_GL_H 1
/* #undef	HAVE_OPENGL_GL_H */

/* #undef	NEED_SUNOS_H */

/* system calls/library functions */

#define	AT_EXIT atexit

#define	HAVE_TZSET 1

#define	HAVE_OPENPTY 1
/* #undef	HAVE_UNIX98_PTYS */

#define	HAVE_SNPRINTF 1
/* #undef	HAVE___SNPRINTF */

#define	HAVE_MMAP 1

/* #undef	HAVE_DYLD */

#define	HAVE_LANGINFO_CODESET 1

#define	HAVE_ALLOCA 1
#define	HAVE_ALLOCA_H 1

#define	HAVE_UNSETENV 1

#define	HAVE_NANOSLEEP 1

/* widths of primitive types */

#define	SIZEOF_INT 4
#define	SIZEOF_LONG 4
#define	SIZEOF_LONG_LONG 8
#define	SIZEOF_VOID_P 4

/* structures */

#define	HAVE_TM_GMTOFF 1
#define	HAVE_TIMEZONE 1

/* typedefs */

/* #undef	size_t */
/* #undef	socklen_t */

#define	squeakInt64 long long

/* architecture */

#define	OS_TYPE "unix"

#define	VM_HOST "i686-pc-linux-gnu"
#define	VM_HOST_CPU "i686"
/* #undef	VM_HOST_VENDOR */
#define	VM_HOST_OS "linux-gnu"
#define	VM_BUILD_STRING "Unix built on "__DATE__ " "__TIME__" Compiler: "__VERSION__

/* #undef	WORDS_BIGENDIAN */
/* #undef	DOUBLE_WORD_ALIGNMENT */

/* damage containment */

/* #undef	DARWIN */

#ifdef NEED_SUNOS_H
# include "sunos.h"
#endif

/* other configured variables */

#define SQ_VERSION "3.9a-7024"
#define VM_VERSION "3.9-7"
#define VM_MODULE_PREFIX ""
/* #undef VM_DLSYM_PREFIX */
#define VM_X11DIR ""

/* avoid dependencies on glibc2.3 */

#define HAVE_FEATURES_H 1

#if defined(HAVE_FEATURES_H)
# include "glibc.h"
#endif

#include <stdint.h>
uint64_t bCounter;
uint64_t mCounter;
uint64_t pCounter;

/* used by UUID plugin: */

#define HAVE_UUID_H 1

#endif /* __sq_config_h */
																																																																					
																																																																				'! !

!ProfileUnixDebugConfig methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/5/2014 10:07'!
executableName
	^ 'ProfileStackDebugVM'! !

!ProfileUnixDebugConfig methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/5/2014 10:06'!
interpreterClass
	^ ProfileStackInterpreter! !

StackEvtAndroidConfig subclass: #MetaStackEvtAndroidConfig
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'UnixMetaStack'!

!MetaStackEvtAndroidConfig methodsFor: 'as yet unclassified' stamp: 'np 9/9/2013 12:34'!
executableName
	^ 'MetaStackVM'! !

!MetaStackEvtAndroidConfig methodsFor: 'as yet unclassified' stamp: 'np 9/9/2013 12:35'!
interpreterClass
	^ MetaStackEvtAndroidInterpreter! !

StackUnixConfig subclass: #MetaStackUnixConfig
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'UnixMetaStack'!

!MetaStackUnixConfig methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/7/2013 10:05'!
executableName
	^ 'MetaStackVM'! !

!MetaStackUnixConfig methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 5/6/2013 16:13'!
interpreterClass
	^ MetaStackInterpreter! !

StackUnixConfig subclass: #ProfileUnixConfig
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'UnixMetaStack'!

ProfileUnixConfig subclass: #ExtendedProfileUnixConfig
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'UnixMetaStack'!

!ExtendedProfileUnixConfig methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/7/2014 18:38'!
configH
	" right now its like  that "
	^ '
#ifndef __sq_config_h
#define __sq_config_h

/* explicit image width */

#define	HAVE_INTERP_H 1

/* package options */

#define	USE_X11 1
#define	USE_X11_GLX 1
/* #undef	USE_QUARTZ */
/* #undef	USE_QUARTZ_CGL */
/* #undef	USE_RFB */

/* libraries */

/* #undef	HAVE_LIBX11 */
/* #undef	HAVE_LIBXEXT */
#define	HAVE_LIBDL 1
/* #undef	HAVE_DYLD */
/* #undef	HAVE_LIBFFI */
/* #undef	HAVE_ICONV */

/* #undef	USE_AUDIO_NONE */
/* #undef	USE_AUDIO_SUN */
/* #undef	USE_AUDIO_NAS */
/* #undef	USE_AUDIO_OSS */
/* #undef	USE_AUDIO_MACOSX */
/* #undef	OSS_DEVICE */

/* header files */

#define	HAVE_UNISTD_H 1
/* #undef	NEED_GETHOSTNAME_P */

#define	HAVE_DIRENT_H 1
/* #undef	HAVE_SYS_NDIR_H */
/* #undef	HAVE_SYS_DIR_H */
/* #undef	HAVE_NDIR_H */
#define	HAVE_DLFCN_H 1
#define	HAVE_ICONV_H 1

#define	HAVE_SYS_TIME_H 1
#define	TIME_WITH_SYS_TIME 1

/* #undef	HAVE_SYS_FILIO_H */

/* #undef	HAVE_SYS_AUDIOIO_H */
/* #undef	HAVE_SUN_AUDIOIO_H */

#define	HAVE_PTY_H 1
/* #undef	HAVE_UTIL_H */
/* #undef	HAVE_LIBUTIL_H */
#define	HAVE_STROPTS_H 1

#define	HAVE_GL_GL_H 1
/* #undef	HAVE_OPENGL_GL_H */

/* #undef	NEED_SUNOS_H */

/* system calls/library functions */

#define	AT_EXIT atexit

#define	HAVE_TZSET 1

#define	HAVE_OPENPTY 1
/* #undef	HAVE_UNIX98_PTYS */

#define	HAVE_SNPRINTF 1
/* #undef	HAVE___SNPRINTF */

#define	HAVE_MMAP 1

/* #undef	HAVE_DYLD */

#define	HAVE_LANGINFO_CODESET 1

#define	HAVE_ALLOCA 1
#define	HAVE_ALLOCA_H 1

#define	HAVE_UNSETENV 1

#define	HAVE_NANOSLEEP 1

/* widths of primitive types */

#define	SIZEOF_INT 4
#define	SIZEOF_LONG 4
#define	SIZEOF_LONG_LONG 8
#define	SIZEOF_VOID_P 4

/* structures */

#define	HAVE_TM_GMTOFF 1
#define	HAVE_TIMEZONE 1

/* typedefs */

/* #undef	size_t */
/* #undef	socklen_t */

#define	squeakInt64 long long

/* architecture */

#define	OS_TYPE "unix"

#define	VM_HOST "i686-pc-linux-gnu"
#define	VM_HOST_CPU "i686"
/* #undef	VM_HOST_VENDOR */
#define	VM_HOST_OS "linux-gnu"
#define	VM_BUILD_STRING "Unix built on "__DATE__ " "__TIME__" Compiler: "__VERSION__

/* #undef	WORDS_BIGENDIAN */
/* #undef	DOUBLE_WORD_ALIGNMENT */

/* damage containment */

/* #undef	DARWIN */

#ifdef NEED_SUNOS_H
# include "sunos.h"
#endif

/* other configured variables */

#define SQ_VERSION "3.9a-7024"
#define VM_VERSION "3.9-7"
#define VM_MODULE_PREFIX ""
/* #undef VM_DLSYM_PREFIX */
#define VM_X11DIR ""

/* avoid dependencies on glibc2.3 */

#define HAVE_FEATURES_H 1

#if defined(HAVE_FEATURES_H)
# include "glibc.h"
#endif

#include <stdint.h>
uint64_t bCounter;
uint64_t mCounter;
uint64_t pCounter;

uint64_t bArrayCounter[255];
uint64_t quickPrimitiveCounter[520];
uint64_t slowPrimitiveCounter[576];

/* used by UUID plugin: */

#define HAVE_UUID_H 1

#endif /* __sq_config_h */
																																																																					
																																																																				'! !

!ExtendedProfileUnixConfig methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/7/2014 16:11'!
executableName
	^ 'ExtendedProfileStackVM'! !

!ExtendedProfileUnixConfig methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/7/2014 16:11'!
interpreterClass
	^ ExtendedProfileStackInterpreter! !

!ProfileUnixConfig methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/7/2014 16:07'!
configH
	" right now its like  that "
	^ '
#ifndef __sq_config_h
#define __sq_config_h

/* explicit image width */

#define	HAVE_INTERP_H 1

/* package options */

#define	USE_X11 1
#define	USE_X11_GLX 1
/* #undef	USE_QUARTZ */
/* #undef	USE_QUARTZ_CGL */
/* #undef	USE_RFB */

/* libraries */

/* #undef	HAVE_LIBX11 */
/* #undef	HAVE_LIBXEXT */
#define	HAVE_LIBDL 1
/* #undef	HAVE_DYLD */
/* #undef	HAVE_LIBFFI */
/* #undef	HAVE_ICONV */

/* #undef	USE_AUDIO_NONE */
/* #undef	USE_AUDIO_SUN */
/* #undef	USE_AUDIO_NAS */
/* #undef	USE_AUDIO_OSS */
/* #undef	USE_AUDIO_MACOSX */
/* #undef	OSS_DEVICE */

/* header files */

#define	HAVE_UNISTD_H 1
/* #undef	NEED_GETHOSTNAME_P */

#define	HAVE_DIRENT_H 1
/* #undef	HAVE_SYS_NDIR_H */
/* #undef	HAVE_SYS_DIR_H */
/* #undef	HAVE_NDIR_H */
#define	HAVE_DLFCN_H 1
#define	HAVE_ICONV_H 1

#define	HAVE_SYS_TIME_H 1
#define	TIME_WITH_SYS_TIME 1

/* #undef	HAVE_SYS_FILIO_H */

/* #undef	HAVE_SYS_AUDIOIO_H */
/* #undef	HAVE_SUN_AUDIOIO_H */

#define	HAVE_PTY_H 1
/* #undef	HAVE_UTIL_H */
/* #undef	HAVE_LIBUTIL_H */
#define	HAVE_STROPTS_H 1

#define	HAVE_GL_GL_H 1
/* #undef	HAVE_OPENGL_GL_H */

/* #undef	NEED_SUNOS_H */

/* system calls/library functions */

#define	AT_EXIT atexit

#define	HAVE_TZSET 1

#define	HAVE_OPENPTY 1
/* #undef	HAVE_UNIX98_PTYS */

#define	HAVE_SNPRINTF 1
/* #undef	HAVE___SNPRINTF */

#define	HAVE_MMAP 1

/* #undef	HAVE_DYLD */

#define	HAVE_LANGINFO_CODESET 1

#define	HAVE_ALLOCA 1
#define	HAVE_ALLOCA_H 1

#define	HAVE_UNSETENV 1

#define	HAVE_NANOSLEEP 1

/* widths of primitive types */

#define	SIZEOF_INT 4
#define	SIZEOF_LONG 4
#define	SIZEOF_LONG_LONG 8
#define	SIZEOF_VOID_P 4

/* structures */

#define	HAVE_TM_GMTOFF 1
#define	HAVE_TIMEZONE 1

/* typedefs */

/* #undef	size_t */
/* #undef	socklen_t */

#define	squeakInt64 long long

/* architecture */

#define	OS_TYPE "unix"

#define	VM_HOST "i686-pc-linux-gnu"
#define	VM_HOST_CPU "i686"
/* #undef	VM_HOST_VENDOR */
#define	VM_HOST_OS "linux-gnu"
#define	VM_BUILD_STRING "Unix built on "__DATE__ " "__TIME__" Compiler: "__VERSION__

/* #undef	WORDS_BIGENDIAN */
/* #undef	DOUBLE_WORD_ALIGNMENT */

/* damage containment */

/* #undef	DARWIN */

#ifdef NEED_SUNOS_H
# include "sunos.h"
#endif

/* other configured variables */

#define SQ_VERSION "3.9a-7024"
#define VM_VERSION "3.9-7"
#define VM_MODULE_PREFIX ""
/* #undef VM_DLSYM_PREFIX */
#define VM_X11DIR ""

/* avoid dependencies on glibc2.3 */

#define HAVE_FEATURES_H 1

#if defined(HAVE_FEATURES_H)
# include "glibc.h"
#endif

#include <stdint.h>
uint64_t bCounter;
uint64_t mCounter;
uint64_t pCounter;

/* used by UUID plugin: */

#define HAVE_UUID_H 1

#endif /* __sq_config_h */
																																																																					
																																																																				'! !

!ProfileUnixConfig methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/7/2014 16:07'!
executableName
	^ 'ProfileStackVM'! !

!ProfileUnixConfig methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/7/2014 16:07'!
interpreterClass
	^ ProfileStackInterpreter! !
MetaStackInterpreter initialize!
